__all__ = ["LoggerEx"]

from java.lang import Object


class LoggerEx(Object):
    """This class is a wrapper around a logger which provides additional
    useful tools.

    To create one, use the newBuilder() function and configure the
    builder.
    """

    def __init__(self):
        pass

    def getLogger(self):
        pass

    def getName(self):
        pass

    @staticmethod
    def log(*args):
        pass
