# coding=utf-8

"""Package information."""

__title__ = "ignition-api"
__description__ = "Ignition Scripting API."
__url__ = "https://github.com/thecesrom/Ignition"
__version__ = "8.1.10"
__cycle__ = ".post7"
__build__ = "2021090812"
__author__ = "César Román"
__author_email__ = "cesar@thecesrom.dev"
__license__ = "MIT"
__copyright__ = "Copyright (C) 2018-2021 César Román"
